@@ -18,15 +18,29 @@ graphics package, and as LaTeX/TikZ source for direct embedding into documents o
 user-tweaking of the generated output.
 
 TODOs:
+ * Store dict entries as tuple keys, e.g. myblock.entries[1,2] rather than recursive dicts => v1.6.0
+ * Preserve comments from read -> write (needs full-line/inline comment separation?)
  * Split writeSLHA into writeSLHA{Blocks,Decays}
  * Identify HERWIG decay matrix element to use in ISAWIG
  * Handle RPV SUSY in ISAWIG
 """
 
 __author__ = "Andy Buckley <andy.buckley@cern.ch"
-__version__ = "1.4.4"
+__version__ = "1.5.0"
 
 
+def _mkdict():
+    """Try to return an empty ordered dict, but fall back to normal dict if necessary"""
+    try:
+        from collections import OrderedDict
+        return OrderedDict()
+    except:
+        try:
+            from ordereddict import OrderedDict
+            return OrderedDict()
+        except:
+            return dict()
+
 def _autotype(var):
     """Automatically convert strings to numerical types if possible."""
     if type(var) is not str:
@@ -65,20 +79,22 @@ class Block(object):
     """
     def __init__(self, name, q=None):
         self.name = name
-        self.entries = {}
+        self.entries = _mkdict()
         self.q = _autotype(q)
 
     def add_entry(self, entry):
         #print entry
         nextparent = self.entries
+        if type(entry) is str:
+            raise Exception("Block entries must be tuples or lists")
         if len(entry) < 2:
-            raise Exception("Block entries must be at least a 2-tuple")
+            raise Exception("Block entry tuples must have at least two entries")
         #print "in", entry
         entry = map(_autotype, entry)
         #print "out", entry
         for e in entry[:-2]:
             if e is not entry[-1]:
-                nextparent = nextparent.setdefault(e, {})
+                nextparent = nextparent.setdefault(e, _mkdict())
         nextparent[entry[-2]] = entry[-1]
         #print self.entries
 
@@ -179,8 +195,8 @@ def readSLHA(spcstr, ignorenobr=False):
     If the ignorenobr parameter is True, do not store decay entries with a
     branching ratio of zero.
     """
-    blocks = {}
-    decays = {}
+    blocks = _mkdict()
+    decays = _mkdict()
     #
     import re
     currentblock = None
@@ -260,14 +276,14 @@ def writeSLHA(blocks, decays, ignorenobr=False, precision=8):
     sep = "   "
     out = ""
     def dict_hier_strs(d, s=""):
-        if type(d) is dict:
-            for k, v in sorted(d.iteritems()):
+        if isinstance(d, dict):
+            for k, v in d.iteritems():
                 for s2 in dict_hier_strs(v, s + sep + _autostr(k)):
                     yield s2
         else:
             yield s + sep + _autostr(d)
     ## Blocks
-    for bname, b in sorted(blocks.iteritems()):
+    for bname, b in blocks.iteritems():
         namestr = b.name
         if b.q is not None:
             namestr += (" Q= " + fmte) % float(b.q)
@@ -276,9 +292,9 @@ def writeSLHA(blocks, decays, ignorenobr=False, precision=8):
             out += sep + s + "\n"
         out += "\n"
     ## Decays
-    for pid, particle in sorted(decays.iteritems()):
+    for pid, particle in decays.iteritems():
         out += ("DECAY %d " + fmte + "\n") % (particle.pid, particle.totalwidth or -1)
-        for d in sorted(particle.decays):
+        for d in particle.decays:
             if d.br > 0.0 or not ignorenobr:
                 products_str = "   ".join(map(str, d.ids))
                 out += sep + fmte % d.br + sep + "%d" % len(d.ids) + sep + products_str + "\n"
@@ -508,8 +524,8 @@ def readISAWIG(isastr, ignorenobr=False):
     branching ratio of zero.
     """
 
-    blocks = {}
-    decays = {}
+    blocks = _mkdict()
+    decays = _mkdict()
     LINES = isastr.splitlines()
 
     def getnextvalidline():
